@@ -3,12 +3,12 @@
 # @author Simon Heybrock
 from __future__ import annotations
 import re
+import inspect
 import warnings
 import datetime
 import dateutil.parser
-from enum import Enum, auto
 import functools
-from typing import List, Union, Any, Dict, Tuple, Protocol
+from typing import overload, List, Union, Any, Dict, Tuple, Protocol, Optional
 import numpy as np
 import scipp as sc
 import h5py
@@ -55,21 +55,6 @@ class NexusStructureError(Exception):
     pass
 
 
-class NX_class(Enum):
-    NXdata = auto()
-    NXdetector = auto()
-    NXdisk_chopper = auto()
-    NXentry = auto()
-    NXevent_data = auto()
-    NXinstrument = auto()
-    NXlog = auto()
-    NXmonitor = auto()
-    NXroot = auto()
-    NXsample = auto()
-    NXsource = auto()
-    NXtransformations = auto()
-
-
 class Attrs:
     """HDF5 attributes.
     """
@@ -320,9 +305,50 @@ class NXobject:
             da.coords['depends_on'] = t if isinstance(t, sc.Variable) else sc.scalar(t)
         return da
 
-    def __getitem__(
-            self,
-            name: NXobjectIndex) -> Union['NXobject', Field, sc.DataArray, sc.Dataset]:
+    def _get_children_by_nx_class(self, select: type) -> Dict[str, 'NXobject']:
+        children = {}
+        for key in self.keys():
+            if (child := self._get_child(key)).attrs.get('NX_class') == select.__name__:
+                children[key] = child
+        return children
+
+    @overload
+    def __getitem__(self, name: str) -> Union['NXobject', Field]:
+        ...
+
+    @overload
+    def __getitem__(self, name: ScippIndex) -> Union[sc.DataArray, sc.Dataset]:
+        ...
+
+    @overload
+    def __getitem__(self, name: type) -> Dict[str, 'NXobject']:
+        ...
+
+    def __getitem__(self, name):
+        """
+        Get a child group or child dataset, a selection of child groups, or load and
+        return the current group.
+
+        Three cases are supported:
+
+        - String name: The child group or child dataset of that name is returned.
+        - Class such as ``NXdata`` or ``NXlog``: A dict containing all direct children
+          with a matching ``NX_class`` attribute are returned.
+        - Scipp-style index: Load the specified slice of the current group, returning
+          a :class:`scipp.DataArray` or :class:`scipp.Dataset`.
+
+        Parameters
+        ----------
+        name:
+            Child name, class, or index.
+
+        Returns
+        -------
+        :
+            Field, group, dict of fields, or loaded data.
+        """
+        if inspect.isclass(name) and issubclass(name, NXobject):
+            return self._get_children_by_nx_class(name)
         return self._get_child(name, use_field_dims=True)
 
     def _getitem(self, index: ScippIndex) -> Union[sc.DataArray, sc.Dataset]:
@@ -366,38 +392,16 @@ class NXobject:
     def items(self) -> List[Tuple[str, Union[Field, 'NXobject']]]:
         return list(zip(self.keys(), self.values()))
 
-    @functools.lru_cache()
-    def by_nx_class(self) -> Dict[NX_class, Dict[str, 'NXobject']]:
-        classes = {name: [] for name in _nx_class_registry()}
-
-        # TODO implement visititems for NXobject and merge the two blocks
-        def _match_nx_class(_, node):
-            if not hasattr(node, 'shape'):
-                if (nx_class := node.attrs.get('NX_class')) is not None:
-                    if not isinstance(nx_class, str):
-                        nx_class = nx_class.decode('UTF-8')
-                    if nx_class in _nx_class_registry():
-                        classes[nx_class].append(node)
-
-        self._group.visititems(_match_nx_class)
-
-        out = {}
-        for nx_class, groups in classes.items():
-            names = [group.name.split('/')[-1] for group in groups]
-            if len(names) != len(set(names)):  # fall back to full path if duplicate
-                names = [group.name for group in groups]
-            out[NX_class[nx_class]] = {n: _make(g) for n, g in zip(names, groups)}
-        return out
-
     @property
-    def nx_class(self) -> NX_class:
+    def nx_class(self) -> Optional[type]:
         """The value of the NX_class attribute of the group.
 
         In case of the subclass NXroot this returns 'NXroot' even if the attribute
         is not actually set. This is to support the majority of all legacy files, which
         do not have this attribute.
         """
-        return NX_class[self.attrs['NX_class']]
+        if (nxclass := self.attrs.get('NX_class')) is not None:
+            return _nx_class_registry().get(nxclass)
 
     @property
     def depends_on(self) -> Union[sc.Variable, sc.DataArray, None]:
@@ -424,9 +428,20 @@ class NXobject:
             dataset.attrs['start'] = str(start.value)
         return Field(dataset, data.dims)
 
-    def create_class(self, name: str, nx_class: NX_class) -> NXobject:
+    def create_class(self, name: str, nx_class: Union[str, type]) -> NXobject:
+        """Create empty HDF5 group with given name and set the NX_class attribute.
+
+        Parameters
+        ----------
+        name:
+            Group name.
+        nx_class:
+            Nexus class, can be a valid string for the NX_class attribute, or a
+            subclass of NXobject, such as NXdata or NXlog.
+        """
         group = self._group.create_group(name)
-        group.attrs['NX_class'] = nx_class.name
+        attr = nx_class if isinstance(nx_class, str) else nx_class.__name__
+        group.attrs['NX_class'] = attr
         return _make(group)
 
     def __setitem__(self, name: str, value: Union[Field, NXobject, DimensionedArray]):
@@ -438,29 +453,34 @@ class NXobject:
         else:
             self.create_field(name, value)
 
+    def __getattr__(self, attr: str) -> Union[Any, 'NXobject']:
+        nxclass = _nx_class_registry().get(f'NX{attr}')
+        if nxclass is None:
+            raise AttributeError(f"'NXobject' object has no attribute {attr}")
+        matches = self[nxclass]
+        if len(matches) == 0:
+            raise NexusStructureError(f"No group with requested NX_class='{nxclass}'")
+        if len(matches) == 1:
+            return next(iter(matches.values()))
+        raise NexusStructureError(f"Multiple keys match {nxclass}, use obj[{nxclass}] "
+                                  f"to obtain all matches instead of obj.{attr}.")
 
-class NXroot(NXobject):
-    """Root of a NeXus file."""
-
-    @property
-    def nx_class(self) -> NX_class:
-        # As an oversight in the NeXus standard and the reference implementation,
-        # the NX_class was never set to NXroot. This applies to essentially all
-        # files in existence before 2016, and files written by other implementations
-        # that were inspired by the reference implementation. We thus hardcode NXroot:
-        return NX_class['NXroot']
-
-
-class NXentry(NXobject):
-    """Entry in a NeXus file."""
-
-
-class NXinstrument(NXobject):
-    """Group of instrument-related information."""
-
-
-class NXtransformations(NXobject):
-    """Group of transformations."""
+    def __dir__(self):
+        keys = super().__dir__()
+        nxclasses = []
+        # Avoiding self.values() since it is more costly, but mainly since there may be
+        # edge cases where creation of Field/NXobject may raise on unrelated children.
+        for name, val in self._group.items():
+            if not hasattr(val, 'shape'):  # not a dataset
+                nxclasses.append(_make(val).nx_class)
+        for key in set(nxclasses):
+            if key is None:
+                continue
+            if key in keys:
+                continue
+            if nxclasses.count(key) == 1:
+                keys.append(key.__name__[2:])
+        return keys
 
 
 def _make(group) -> NXobject:
@@ -469,20 +489,19 @@ def _make(group) -> NXobject:
     return group  # Return underlying (h5py) group
 
 
+class NXroot(NXobject):
+    """Root of a NeXus file."""
+
+    @property
+    def nx_class(self) -> type:
+        # As an oversight in the NeXus standard and the reference implementation,
+        # the NX_class was never set to NXroot. This applies to essentially all
+        # files in existence before 2016, and files written by other implementations
+        # that were inspired by the reference implementation. We thus hardcode NXroot:
+        return NXroot
+
+
 @functools.lru_cache()
 def _nx_class_registry():
-    from .nxevent_data import NXevent_data
-    from .nxdata import NXdata
-    from .nxdetector import NXdetector
-    from .nxdisk_chopper import NXdisk_chopper
-    from .nxlog import NXlog
-    from .nxmonitor import NXmonitor
-    from .nxsample import NXsample
-    from .nxsource import NXsource
-    return {
-        cls.__name__: cls
-        for cls in [
-            NXroot, NXentry, NXevent_data, NXlog, NXmonitor, NXdata, NXdetector,
-            NXsample, NXsource, NXdisk_chopper, NXinstrument, NXtransformations
-        ]
-    }
+    from . import nexus_classes
+    return dict(inspect.getmembers(nexus_classes, inspect.isclass))
