@@ -5,7 +5,9 @@ import h5py
 import numpy as np
 import pytest
 import scipp as sc
-from scippnexus import Field, NXroot, NX_class
+from scippnexus import (Field, NXroot, NXentry, NXmonitor, NXlog, NXevent_data,
+                        NXdetector)
+from scippnexus import NexusStructureError
 
 # representative sample of UTF-8 test strings from
 # https://www.w3.org/2001/06/utf-8-test/UTF-8-demo.html
@@ -21,17 +23,22 @@ def nxroot(request):
     """Yield NXroot containing a single NXentry named 'entry'"""
     with h5py.File('dummy.nxs', mode='w', driver="core", backing_store=False) as f:
         root = NXroot(f)
-        root.create_class('entry', NX_class.NXentry)
+        root.create_class('entry', NXentry)
         yield root
 
 
 def test_nxobject_root(nxroot):
-    assert nxroot.nx_class == NX_class.NXroot
+    assert nxroot.nx_class == NXroot
     assert set(nxroot.keys()) == {'entry'}
 
 
 def test_nxobject_create_class_creates_keys(nxroot):
-    nxroot.create_class('log', NX_class.NXlog)
+    nxroot.create_class('log', NXlog)
+    assert set(nxroot.keys()) == {'entry', 'log'}
+
+
+def test_nxobject_create_class_with_string_nx_class(nxroot):
+    nxroot.create_class('log', 'NXlog')
     assert set(nxroot.keys()) == {'entry', 'log'}
 
 
@@ -40,21 +47,21 @@ def test_nxobject_items(nxroot):
     assert len(items) == 1
     name, entry = items[0]
     assert name == 'entry'
-    entry.create_class('monitor', NX_class.NXmonitor)
-    entry.create_class('log', NX_class.NXlog)
+    entry.create_class('monitor', NXmonitor)
+    entry.create_class('log', NXlog)
     assert {k: v.nx_class
             for k, v in entry.items()} == {
-                'log': NX_class.NXlog,
-                'monitor': NX_class.NXmonitor
+                'log': NXlog,
+                'monitor': NXmonitor
             }
 
 
 def test_nxobject_entry(nxroot):
     entry = nxroot['entry']
-    assert entry.nx_class == NX_class.NXentry
-    entry.create_class('events_0', NX_class.NXevent_data)
-    entry.create_class('events_1', NX_class.NXevent_data)
-    entry.create_class('log', NX_class.NXlog)
+    assert entry.nx_class == NXentry
+    entry.create_class('events_0', NXevent_data)
+    entry.create_class('events_1', NXevent_data)
+    entry.create_class('log', NXlog)
     assert set(entry.keys()) == {'events_0', 'events_1', 'log'}
 
 
@@ -66,10 +73,10 @@ def test_nxobject_log(nxroot):
                           sc.array(dims=['time'], unit='s', values=[4.4, 5.5, 6.6]).to(
                               unit='ns', dtype='int64')
                       })
-    log = nxroot['entry'].create_class('log', NX_class.NXlog)
+    log = nxroot['entry'].create_class('log', NXlog)
     log['value'] = da.data
     log['time'] = da.coords['time'] - sc.epoch(unit='ns')
-    assert log.nx_class == NX_class.NXlog
+    assert log.nx_class == NXlog
     assert sc.identical(log[...], da)
 
 
@@ -81,10 +88,10 @@ def test_nxlog_length_1(nxroot):
             sc.epoch(unit='ns') +
             sc.array(dims=['time'], unit='s', values=[4.4]).to(unit='ns', dtype='int64')
         })
-    log = nxroot['entry'].create_class('log', NX_class.NXlog)
+    log = nxroot['entry'].create_class('log', NXlog)
     log['value'] = da.data
     log['time'] = da.coords['time'] - sc.epoch(unit='ns')
-    assert log.nx_class == NX_class.NXlog
+    assert log.nx_class == NXlog
     assert sc.identical(log[...], da)
 
 
@@ -96,7 +103,7 @@ def test_nxlog_length_1_two_dims_no_time_squeezes_all_dims(nxroot):
             sc.epoch(unit='ns') +
             sc.array(dims=['time'], unit='s', values=[4.4]).to(unit='ns', dtype='int64')
         })
-    log = nxroot['entry'].create_class('log', NX_class.NXlog)
+    log = nxroot['entry'].create_class('log', NXlog)
     log['value'] = da.data
     assert sc.identical(log[...], sc.DataArray(sc.scalar(1.1)))
 
@@ -109,7 +116,7 @@ def test_nxlog_length_1_two_dims_with_time_squeezes_inner_dim(nxroot):
             sc.epoch(unit='ns') +
             sc.array(dims=['time'], unit='s', values=[4.4]).to(unit='ns', dtype='int64')
         })
-    log = nxroot['entry'].create_class('log', NX_class.NXlog)
+    log = nxroot['entry'].create_class('log', NXlog)
     log['value'] = da.data
     log['time'] = da.coords['time'] - sc.epoch(unit='ns')
     assert sc.identical(log[...], da['ignored', 0])
@@ -123,7 +130,7 @@ def test_nxlog_axes_replaces_time_dim(nxroot):
             sc.epoch(unit='ns') +
             sc.array(dims=['time'], unit='s', values=[4.4]).to(unit='ns', dtype='int64')
         })
-    log = nxroot['entry'].create_class('log', NX_class.NXlog)
+    log = nxroot['entry'].create_class('log', NXlog)
     log.attrs['axes'] = ['yy', 'xx']
     log['value'] = da.data
     log['time'] = da.coords['time'] - sc.epoch(unit='ns')
@@ -140,7 +147,7 @@ def test_nxlog_three_dims_with_time_of_length_1(nxroot):
             sc.epoch(unit='ns') +
             sc.array(dims=['time'], unit='s', values=[4.4]).to(unit='ns', dtype='int64')
         })
-    log = nxroot['entry'].create_class('log', NX_class.NXlog)
+    log = nxroot['entry'].create_class('log', NXlog)
     log['value'] = da.data
     log['time'] = da.coords['time'] - sc.epoch(unit='ns')
     loaded = log[...]
@@ -153,15 +160,15 @@ def test_nxlog_three_dims_with_time_of_length_1(nxroot):
 def test_nxlog_with_shape_0(nxroot):
     da = sc.DataArray(sc.ones(dims=['time', 'ignored'], shape=(0, 1)),
                       coords={'time': sc.ones(dims=['time'], shape=(0, ), unit='s')})
-    log = nxroot['entry'].create_class('log', NX_class.NXlog)
+    log = nxroot['entry'].create_class('log', NXlog)
     log['value'] = da.data
     log['time'] = da.coords['time']
     assert sc.identical(log[...], da['ignored', 0])
 
 
 def test_nxobject_event_data(nxroot):
-    event_data = nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
-    assert event_data.nx_class == NX_class.NXevent_data
+    event_data = nxroot['entry'].create_class('events_0', NXevent_data)
+    assert event_data.nx_class == NXevent_data
 
 
 def test_nxobject_getting_item_that_does_not_exists_raises_KeyError(nxroot):
@@ -170,9 +177,9 @@ def test_nxobject_getting_item_that_does_not_exists_raises_KeyError(nxroot):
 
 
 def test_nxobject_name_property_is_full_path(nxroot):
-    nxroot.create_class('monitor', NX_class.NXmonitor)
-    nxroot['entry'].create_class('log', NX_class.NXlog)
-    nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
+    nxroot.create_class('monitor', NXmonitor)
+    nxroot['entry'].create_class('log', NXlog)
+    nxroot['entry'].create_class('events_0', NXevent_data)
     assert nxroot.name == '/'
     assert nxroot['monitor'].name == '/monitor'
     assert nxroot['entry'].name == '/entry'
@@ -181,44 +188,33 @@ def test_nxobject_name_property_is_full_path(nxroot):
 
 
 def test_nxobject_grandchild_can_be_accessed_using_path(nxroot):
-    nxroot['entry'].create_class('log', NX_class.NXlog)
+    nxroot['entry'].create_class('log', NXlog)
     assert nxroot['entry/log'].name == '/entry/log'
     assert nxroot['/entry/log'].name == '/entry/log'
 
 
-def test_nxobject_by_nx_class_of_root_contains_everything(nxroot):
-    nxroot.create_class('monitor', NX_class.NXmonitor)
-    nxroot['entry'].create_class('log', NX_class.NXlog)
-    nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
-    nxroot['entry'].create_class('events_1', NX_class.NXevent_data)
-    classes = nxroot.by_nx_class()
-    assert list(classes[NX_class.NXentry]) == ['entry']
-    assert list(classes[NX_class.NXmonitor]) == ['monitor']
-    assert list(classes[NX_class.NXlog]) == ['log']
-    assert set(classes[NX_class.NXevent_data]) == {'events_0', 'events_1'}
-
-
-def test_nxobject_by_nx_class_contains_only_children(nxroot):
-    nxroot.create_class('monitor', NX_class.NXmonitor)
-    nxroot['entry'].create_class('log', NX_class.NXlog)
-    nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
-    nxroot['entry'].create_class('events_1', NX_class.NXevent_data)
-    classes = nxroot['entry'].by_nx_class()
-    assert list(classes[NX_class.NXentry]) == []
-    assert list(classes[NX_class.NXmonitor]) == []
-    assert list(classes[NX_class.NXlog]) == ['log']
-    assert set(classes[NX_class.NXevent_data]) == set(['events_0', 'events_1'])
+def test_nxobject_getitem_by_class(nxroot):
+    nxroot.create_class('monitor', NXmonitor)
+    nxroot['entry'].create_class('log', NXlog)
+    nxroot['entry'].create_class('events_0', NXevent_data)
+    nxroot['entry'].create_class('events_1', NXevent_data)
+    assert list(nxroot[NXentry]) == ['entry']
+    assert list(nxroot[NXmonitor]) == ['monitor']
+    assert list(nxroot['entry'][NXmonitor]) == []  # not nested
+    assert list(nxroot[NXlog]) == []  # nested
+    assert list(nxroot['entry'][NXlog]) == ['log']
+    assert set(nxroot['entry'][NXevent_data]) == {'events_0', 'events_1'}
 
 
 def test_nxobject_dataset_items_are_returned_as_Field(nxroot):
-    events = nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
+    events = nxroot['entry'].create_class('events_0', NXevent_data)
     events['event_time_offset'] = sc.arange('event', 5)
     field = nxroot['entry/events_0/event_time_offset']
     assert isinstance(field, Field)
 
 
 def test_field_properties(nxroot):
-    events = nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
+    events = nxroot['entry'].create_class('events_0', NXevent_data)
     events['event_time_offset'] = sc.arange('event', 6, dtype='int64', unit='ns')
     field = nxroot['entry/events_0/event_time_offset']
     assert field.dtype == 'int64'
@@ -228,7 +224,7 @@ def test_field_properties(nxroot):
 
 
 def test_field_dim_labels(nxroot):
-    events = nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
+    events = nxroot['entry'].create_class('events_0', NXevent_data)
     events['event_time_offset'] = sc.arange('ignored', 2)
     events['event_time_zero'] = sc.arange('ignored', 2)
     events['event_index'] = sc.arange('ignored', 2)
@@ -238,7 +234,7 @@ def test_field_dim_labels(nxroot):
     assert event_data['event_time_zero'].dims == ['pulse']
     assert event_data['event_index'].dims == ['pulse']
     assert event_data['event_id'].dims == ['event']
-    log = nxroot['entry'].create_class('log', NX_class.NXlog)
+    log = nxroot['entry'].create_class('log', NXlog)
     log['value'] = sc.arange('ignored', 2)
     log['time'] = sc.arange('ignored', 2)
     assert log['time'].dims == ['time']
@@ -246,7 +242,7 @@ def test_field_dim_labels(nxroot):
 
 
 def test_field_unit_is_none_if_no_units_attribute(nxroot):
-    log = nxroot.create_class('log', NX_class.NXlog)
+    log = nxroot.create_class('log', NXlog)
     log['value'] = sc.arange('ignored', 2, unit=None)
     log['time'] = sc.arange('ignored', 2)
     assert log.unit is None
@@ -356,7 +352,7 @@ def create_event_data_ids_1234(group):
 
 
 def test_negative_event_index_converted_to_num_event(nxroot):
-    event_data = nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
+    event_data = nxroot['entry'].create_class('events_0', NXevent_data)
     create_event_data_ids_1234(event_data)
     events = nxroot['entry/events_0'][...]
     assert events.bins.size().values[2] == 3
@@ -364,7 +360,7 @@ def test_negative_event_index_converted_to_num_event(nxroot):
 
 
 def test_bad_event_index_raises_IndexError(nxroot):
-    event_data = nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
+    event_data = nxroot['entry'].create_class('events_0', NXevent_data)
     event_data['event_id'] = sc.array(dims=[''], unit=None, values=[1, 2, 4, 1, 2])
     event_data['event_time_offset'] = sc.array(dims=[''], unit='s', values=[0, 0, 0, 0])
     event_data['event_time_zero'] = sc.array(dims=[''], unit='s', values=[1, 2, 3, 4])
@@ -382,7 +378,7 @@ def create_event_data_without_event_id(group):
 
 
 def test_event_data_without_event_id_can_be_loaded(nxroot):
-    event_data = nxroot['entry'].create_class('events_0', NX_class.NXevent_data)
+    event_data = nxroot['entry'].create_class('events_0', NXevent_data)
     create_event_data_without_event_id(event_data)
     da = event_data[...]
     assert len(da.bins.coords) == 1
@@ -390,8 +386,39 @@ def test_event_data_without_event_id_can_be_loaded(nxroot):
 
 
 def test_event_mode_monitor_without_event_id_can_be_loaded(nxroot):
-    monitor = nxroot['entry'].create_class('monitor', NX_class.NXmonitor)
+    monitor = nxroot['entry'].create_class('monitor', NXmonitor)
     create_event_data_without_event_id(monitor)
     da = monitor[...]
     assert len(da.bins.coords) == 1
     assert 'event_time_offset' in da.bins.coords
+
+
+def test___getattr__for_unique_child_groups(nxroot):
+    entry = nxroot['entry']
+    with pytest.raises(NexusStructureError):
+        entry.log
+    entry.create_class('log1', NXlog)
+    log = entry.log
+    assert log.nx_class == NXlog
+    assert log.name == '/entry/log1'
+    assert isinstance(log, NXlog)
+    entry.create_class('log2', NXlog)
+    with pytest.raises(NexusStructureError):
+        entry.log
+
+
+def test___dir__(nxroot):
+    entry = nxroot['entry']
+    assert 'log' not in entry.__dir__()
+    entry.create_class('log1', NXlog)
+    assert 'log' in entry.__dir__()
+    entry.create_class('log2', NXlog)
+    assert 'log' not in entry.__dir__()
+
+
+def test___dir__includes_non_dynamic_properties(nxroot):
+    entry = nxroot['entry']
+    det = entry.create_class('det', NXdetector)
+    det.create_class('events', NXevent_data)
+    # Ensure we are not replacing __dir__ but adding to it
+    assert 'unit' in det.__dir__()
