@@ -1,7 +1,7 @@
 import h5py
 import numpy as np
 import scipp as sc
-from scippnexus import NXroot, NX_class
+from scippnexus import NXroot, NXentry, NXdetector, NXtransformations, NXlog
 from scippnexus import nxtransformations
 import pytest
 
@@ -11,7 +11,7 @@ def nxroot(request):
     """Yield NXroot containing a single NXentry named 'entry'"""
     with h5py.File('dummy.nxs', mode='w', driver="core", backing_store=False) as f:
         root = NXroot(f)
-        root.create_class('entry', NX_class.NXentry)
+        root.create_class('entry', NXentry)
         yield root
 
 
@@ -20,7 +20,7 @@ def create_detector(group):
     detector_numbers = sc.array(dims=['xx', 'yy'],
                                 unit=None,
                                 values=np.array([[1, 2], [3, 4]]))
-    detector = group.create_class('detector_0', NX_class.NXdetector)
+    detector = group.create_class('detector_0', NXdetector)
     detector.create_field('detector_number', detector_numbers)
     detector.create_field('data', data)
     return detector
@@ -29,8 +29,7 @@ def create_detector(group):
 def test_Transformation_with_single_value(nxroot):
     detector = create_detector(nxroot)
     detector.create_field('depends_on', sc.scalar('/detector_0/transformations/t1'))
-    transformations = detector.create_class('transformations',
-                                            NX_class.NXtransformations)
+    transformations = detector.create_class('transformations', NXtransformations)
     value = sc.scalar(6.5, unit='mm')
     offset = sc.spatial.translation(value=[1, 2, 3], unit='mm')
     vector = sc.vector(value=[0, 0, 1])
@@ -55,8 +54,7 @@ def test_Transformation_with_single_value(nxroot):
 def test_chain_with_single_values_and_different_unit(nxroot):
     detector = create_detector(nxroot)
     detector.create_field('depends_on', sc.scalar('/detector_0/transformations/t1'))
-    transformations = detector.create_class('transformations',
-                                            NX_class.NXtransformations)
+    transformations = detector.create_class('transformations', NXtransformations)
     value = sc.scalar(6.5, unit='mm')
     offset = sc.spatial.translation(value=[1, 2, 3], unit='mm')
     vector = sc.vector(value=[0, 0, 1])
@@ -82,8 +80,7 @@ def test_chain_with_single_values_and_different_unit(nxroot):
 def test_Transformation_with_multiple_values(nxroot):
     detector = create_detector(nxroot)
     detector.create_field('depends_on', sc.scalar('/detector_0/transformations/t1'))
-    transformations = detector.create_class('transformations',
-                                            NX_class.NXtransformations)
+    transformations = detector.create_class('transformations', NXtransformations)
     log = sc.DataArray(
         sc.array(dims=['time'], values=[1.1, 2.2], unit='m'),
         coords={'time': sc.array(dims=['time'], values=[11, 22], unit='s')})
@@ -93,7 +90,7 @@ def test_Transformation_with_multiple_values(nxroot):
     t = log * vector
     t.data = sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit)
     expected = t * offset
-    value = transformations.create_class('t1', NX_class.NXlog)
+    value = transformations.create_class('t1', NXlog)
     value['time'] = log.coords['time'] - sc.epoch(unit='ns')
     value['value'] = log.data
     value.attrs['depends_on'] = '.'
@@ -113,8 +110,7 @@ def test_Transformation_with_multiple_values(nxroot):
 def test_chain_with_multiple_values(nxroot):
     detector = create_detector(nxroot)
     detector.create_field('depends_on', sc.scalar('/detector_0/transformations/t1'))
-    transformations = detector.create_class('transformations',
-                                            NX_class.NXtransformations)
+    transformations = detector.create_class('transformations', NXtransformations)
     log = sc.DataArray(
         sc.array(dims=['time'], values=[1.1, 2.2], unit='m'),
         coords={'time': sc.array(dims=['time'], values=[11, 22], unit='s')})
@@ -123,7 +119,7 @@ def test_chain_with_multiple_values(nxroot):
     vector = sc.vector(value=[0, 0, 1])
     t = log * vector
     t.data = sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit)
-    value1 = transformations.create_class('t1', NX_class.NXlog)
+    value1 = transformations.create_class('t1', NXlog)
     value1['time'] = log.coords['time'] - sc.epoch(unit='ns')
     value1['value'] = log.data
     value1.attrs['depends_on'] = 't2'
@@ -131,7 +127,7 @@ def test_chain_with_multiple_values(nxroot):
     value1.attrs['offset'] = offset.values
     value1.attrs['offset_units'] = str(offset.unit)
     value1.attrs['vector'] = vector.value
-    value2 = transformations.create_class('t2', NX_class.NXlog)
+    value2 = transformations.create_class('t2', NXlog)
     value2['time'] = log.coords['time'] - sc.epoch(unit='ns')
     value2['value'] = log.data
     value2.attrs['depends_on'] = '.'
@@ -146,8 +142,7 @@ def test_chain_with_multiple_values(nxroot):
 def test_chain_with_multiple_values_and_different_time_unit(nxroot):
     detector = create_detector(nxroot)
     detector.create_field('depends_on', sc.scalar('/detector_0/transformations/t1'))
-    transformations = detector.create_class('transformations',
-                                            NX_class.NXtransformations)
+    transformations = detector.create_class('transformations', NXtransformations)
     # Making sure to not use nanoseconds since that is used internally and may thus
     # mask bugs.
     log = sc.DataArray(
@@ -158,7 +153,7 @@ def test_chain_with_multiple_values_and_different_time_unit(nxroot):
     vector = sc.vector(value=[0, 0, 1])
     t = log * vector
     t.data = sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit)
-    value1 = transformations.create_class('t1', NX_class.NXlog)
+    value1 = transformations.create_class('t1', NXlog)
     value1['time'] = log.coords['time'] - sc.epoch(unit='us')
     value1['value'] = log.data
     value1.attrs['depends_on'] = 't2'
@@ -166,7 +161,7 @@ def test_chain_with_multiple_values_and_different_time_unit(nxroot):
     value1.attrs['offset'] = offset.values
     value1.attrs['offset_units'] = str(offset.unit)
     value1.attrs['vector'] = vector.value
-    value2 = transformations.create_class('t2', NX_class.NXlog)
+    value2 = transformations.create_class('t2', NXlog)
     value2['time'] = log.coords['time'].to(unit='ms') - sc.epoch(unit='ms')
     value2['value'] = log.data
     value2.attrs['depends_on'] = '.'
